@@ -28,7 +28,7 @@
 """Basic neural mass backend class plus derivations of it.
 
 This module includes the base circuit class that manages the set-up and simulation of population networks. Additionally,
-it features various sub-classes that act as circuit constructors and allow to build circuits based on different input
+it features various subclasses that act as circuit constructors and allow to build circuits based on different input
 arguments. For more detailed descriptions, see the respective docstrings.
 
 """
@@ -137,9 +137,10 @@ class CircuitTemplate(AbstractBaseTemplate):
         self._depth = self._get_hierarchy_depth()
         self._vectorization_labels = {}
         self._vectorization_indices = {}
+        self._state_var_indices = {}
 
     def to_yaml(self, path, **kwargs) -> None:
-        """Short hand to save the `CircuitTemplate` to a yaml file. After that call, either a new YAML file has been
+        """Shorthand to save the `CircuitTemplate` to a yaml file. After that call, either a new YAML file has been
         created including all template definitions required to reconstruct the `CircuitTemplate` instance, or additional
         operators have been added to the already existing YAML file.
 
@@ -263,9 +264,11 @@ class CircuitTemplate(AbstractBaseTemplate):
             target_nodes = self.get_nodes(node_identifier=node, var_identifier=(op, var))
             if not target_nodes:
                 warn(PyRatesWarning(f'Variable {var} has not been found on operator {op} of node {node[0]}.'))
-            for n in target_nodes:
+            n_nodes = len(target_nodes)
+            for i, n in enumerate(target_nodes):
                 node_temp = deepcopy(self.get_node_template(n))
-                node_temp.update_var(op=op, var=var, val=val)
+                val_tmp = val[i] if hasattr(val, 'shape') and val.shape[0] == n_nodes else val
+                node_temp.update_var(op=op, var=var, val=val_tmp)
                 self.add_node_template(n, template=node_temp)
 
         # updates to edge variable values
@@ -276,7 +279,7 @@ class CircuitTemplate(AbstractBaseTemplate):
         return self
 
     def add_edges_from_matrix(self, source_var: str, target_var: str, nodes: list, weight=None, template=None,
-                              edge_attr: dict = None) -> None:
+                              edge_attr: dict = None, min_weight: float = 1e-6) -> None:
         """Adds all possible edges between the `source_var` and `target_var` of all passed `nodes`. `Weight` and `Delay`
         need to be arrays containing scalars for each of those edges.
 
@@ -295,6 +298,8 @@ class CircuitTemplate(AbstractBaseTemplate):
             Can be link to edge template that should be used for each edge.
         edge_attr
             Additional edge attributes. Can either be N x N matrices or other scalars/objects.
+        min_weight
+            Minimum absolute value a weight needs to have in order to be implemented as an edge.
 
         Returns
         -------
@@ -310,7 +315,7 @@ class CircuitTemplate(AbstractBaseTemplate):
 
         # weights and delays
         if weight is None:
-            weight = 1.0
+            weight = np.ones((len(nodes), len(nodes)))
         edge_attributes = {'weight': weight}
 
         # add rest of the attributes
@@ -330,23 +335,25 @@ class CircuitTemplate(AbstractBaseTemplate):
         for i, source in enumerate(nodes):
             for j, target in enumerate(nodes):
 
-                if source not in self.nodes:
-                    raise ValueError(f'Node {source} is not defined on this CircuitTemplate instance.')
-                if target not in self.nodes:
-                    raise ValueError(f'Node {target} is not defined on this CircuitTemplate instance.')
+                if np.abs(weight[j, i]) > min_weight:
 
-                edge_attributes_tmp = {}
+                    if source not in self.nodes:
+                        raise ValueError(f'Node {source} is not defined on this CircuitTemplate instance.')
+                    if target not in self.nodes:
+                        raise ValueError(f'Node {target} is not defined on this CircuitTemplate instance.')
 
-                # extract edge attribute value from matrices
-                for key, attr in matrix_attributes.items():
-                    edge_attributes_tmp[key] = attr[j, i]
+                    edge_attributes_tmp = {}
 
-                # add remaining attributes
-                edge_attributes_tmp.update(edge_attributes.copy())
+                    # extract edge attribute value from matrices
+                    for key, attr in matrix_attributes.items():
+                        edge_attributes_tmp[key] = attr[j, i]
 
-                # add edge to list
-                source_key, target_key = f"{source}/{source_var}", f"{target}/{target_var}"
-                edges.append((source_key, target_key, template, edge_attributes_tmp))
+                    # add remaining attributes
+                    edge_attributes_tmp.update(edge_attributes.copy())
+
+                    # add edge to list
+                    source_key, target_key = f"{source}/{source_var}", f"{target}/{target_var}"
+                    edges.append((source_key, target_key, template, edge_attributes_tmp))
 
         # add edges to network
         self.update_template(edges=edges, in_place=True)
@@ -499,7 +506,7 @@ class CircuitTemplate(AbstractBaseTemplate):
 
     def get_run_func(self, func_name: str, step_size: float, inputs: Optional[dict] = None, backend: str = None,
                      vectorize: bool = True, verbose: bool = True, clear: bool = False, in_place: bool = True, **kwargs
-                     ) -> Tuple[Callable, tuple, tuple]:
+                     ) -> Tuple[Callable, tuple, tuple, dict]:
         """Generate a function that evaluates the vector field of the dynamical system represented by this
         `CircuitTemplate` instance.
 
@@ -540,8 +547,9 @@ class CircuitTemplate(AbstractBaseTemplate):
 
         Returns
         -------
-        Tuple[Callable, tuple, tuple]
-            The vector field evaluation function, all its positional arguments, and the argument keys.
+        Tuple[Callable, tuple, tuple, dict]
+            The vector field evaluation function, all its positional arguments, the argument keys, and the indices of
+            the different state variables in the state vector.
 
         """
 
@@ -560,14 +568,16 @@ class CircuitTemplate(AbstractBaseTemplate):
                   vectorize=vectorize, **kwargs)
 
         # generate the run function
-        func, args, arg_names = net._ir.get_run_func(func_name=func_name, step_size=step_size, **kwargs)
+        func, args, arg_names, state_var_indices = net._ir.get_run_func(func_name=func_name, step_size=step_size,
+                                                                        **kwargs)
+        self._state_var_indices = state_var_indices
 
         # clear the network temporary files
         if clear:
             net.clear()
         self._ir = net._ir
 
-        return func, args, arg_names
+        return func, args, arg_names, state_var_indices
 
     def apply(self, adaptive_steps: bool = None, label: str = None, node_values: dict = None, edge_values: dict = None,
               vectorize: bool = True, verbose: bool = True, **kwargs) -> None:
@@ -633,8 +643,6 @@ class CircuitTemplate(AbstractBaseTemplate):
         edges = []
         for (source, target, template, _), values in edge_col.items():
 
-            values = deepcopy(values)
-
             # update edge template default values with passed edge values,
             if (source, target) in edge_values:
                 values.update(edge_values[(source, target)])
@@ -938,6 +946,26 @@ class CircuitTemplate(AbstractBaseTemplate):
             idx = 0
         return self._edge_map[(source, target, idx)]
 
+    def get_var(self, var: str) -> tuple:
+        """
+
+        Parameters
+        ----------
+        var
+            Identifier of variable in the network.
+
+        Returns
+        -------
+        tuple
+            2-entry tuple: (1) Backend variable, (2) index of requested variable in the backend variable.
+        """
+
+        *node, op, var = var.split('/')
+        nodes = self.get_nodes(node, var_identifier=(op, var))
+        backend_var = self._ir.get_var(f"{nodes[0]}/{op}/{var}")
+        idx = [self._get_var_idx(f"{n}/{op}/{var}") for n in nodes]
+        return backend_var, idx
+
     def get_variable_positions(self, outputs: Union[dict, str]) -> tuple:
         """Finds the indices of variables in the system state vector as well as the backend variables from which the
         variables can be extracted via the indices.
@@ -956,7 +984,6 @@ class CircuitTemplate(AbstractBaseTemplate):
 
         out_map = {}
         out_vars = {}
-        indices = self._vectorization_indices
 
         if type(outputs) is dict:
             for key, out in outputs.items():
@@ -969,9 +996,9 @@ class CircuitTemplate(AbstractBaseTemplate):
                 if len(target_nodes) == 1:
 
                     # extract index for single output node
-                    backend_key = self._relabel_var(f"{target_nodes[0]}/{out_op}/{out_var}", self._vectorization_labels)
-                    idx = indices[target_nodes[0]]
-                    out_map[key] = [idx]
+                    var_key = f"{target_nodes[0]}/{out_op}/{out_var}"
+                    backend_key = self._relabel_var(var_key, self._vectorization_labels)
+                    out_map[key] = self._get_var_idx(var_key)
                     out_vars[key] = backend_key
 
                 elif target_nodes:
@@ -979,11 +1006,10 @@ class CircuitTemplate(AbstractBaseTemplate):
                     # extract index for multiple output nodes
                     out_map[key] = {}
                     for t in target_nodes:
-                        key2 = f"{t}/{out_op}/{out_var}"
-                        backend_key = self._relabel_var(key2, self._vectorization_labels)
-                        idx = indices[t]
-                        out_map[key][key2] = [idx]
-                        out_vars[key2] = backend_key
+                        var_key = f"{t}/{out_op}/{out_var}"
+                        backend_key = self._relabel_var(var_key, self._vectorization_labels)
+                        out_map[key][var_key] = self._get_var_idx(var_key)
+                        out_vars[var_key] = backend_key
 
         else:
 
@@ -995,8 +1021,7 @@ class CircuitTemplate(AbstractBaseTemplate):
             for t in target_nodes:
                 key = f"{t}/{out_op}/{out_var}"
                 backend_key = self._relabel_var(key, self._vectorization_labels)
-                idx = indices[t]
-                out_map[key] = [idx]
+                out_map[key] = self._get_var_idx(key)
                 out_vars[key] = backend_key
 
         return out_map, out_vars
@@ -1023,6 +1048,14 @@ class CircuitTemplate(AbstractBaseTemplate):
         network equations that this `CircuitTemplate` contains."""
         return self._ir.graph
 
+    def _get_var_idx(self, var: str) -> list:
+        idx = self._vectorization_indices[var]
+        try:
+            *n, o, v = var.split('/')
+            return np.arange(*self._state_var_indices[v])[idx]
+        except KeyError:
+            return idx
+
     def _apply_nodes(self, node_keys: list, values: dict, vectorize: bool = True) -> dict:
         nodes = {}
         indices = {}
@@ -1030,9 +1063,13 @@ class CircuitTemplate(AbstractBaseTemplate):
         for node in node_keys:
             updates = values[node] if node in values else {}
             node_template = self.get_node_template(node)
-            node_ir, label_map_tmp = node_template.apply(values=updates, label=node, vectorize=vectorize)
+            node_ir, label_map_tmp, var_ranges = node_template.apply(values=updates, label=node, vectorize=vectorize)
             nodes[node_ir.label] = node_ir
-            indices[node] = node_ir.length - 1
+            new_ops, orig_ops = list(label_map_tmp.values()), list(label_map_tmp.keys())
+            for (op, var), (start, stop) in var_ranges.items():
+                if op in new_ops:
+                    op = orig_ops[new_ops.index(op)]
+                indices[f"{node}/{op}/{var}"] = list(np.arange(start, stop))
             for key, val in label_map_tmp.items():
                 label_map[f"{node}/{key}"] = f"{node_ir.label}/{val}"
             else:
@@ -1177,18 +1214,14 @@ class CircuitTemplate(AbstractBaseTemplate):
 
             edge_dict = deepcopy(edge_dict)
 
-            # extract variable info
-            *s_node, s_op, s_var = source.split('/')
-            *t_node, t_op, t_var = target.split('/')
-            s_node = '/'.join(s_node)
-            t_node = '/'.join(t_node)
-
             # relabel variables according to variable map (accounting for vectorization)
             source_new = self._relabel_var(source, label_map)
             target_new = self._relabel_var(target, label_map)
 
-            s_idx = indices[s_node]
-            t_idx = indices[t_node]
+            # extract indices of source and target variables in their respective vectors
+            s_idx = indices[source]
+            t_idx = indices[target]
+            edge_len = len(s_idx)
 
             # group edges that connect the same vectorized node variables via the same edge templates
             if (source_new, target_new, template, delayed) in edge_col:
@@ -1196,12 +1229,11 @@ class CircuitTemplate(AbstractBaseTemplate):
                 # extend edge dict by edge variables
                 base_dict = edge_col[(source_new, target_new, template, delayed)]
                 for key, val in edge_dict.items():
-                    try:
-                        base_dict[key].append(val)
-                    except AttributeError:
-                        base_dict[key] = [base_dict[key], val]
-                base_dict['source_idx'].append(s_idx)
-                base_dict['target_idx'].append(t_idx)
+                    if type(val) is not str:
+                        val = [val] * edge_len
+                        base_dict[key].extend(val)
+                base_dict['source_idx'].extend(s_idx)
+                base_dict['target_idx'].extend(t_idx)
 
             else:
 
@@ -1210,9 +1242,9 @@ class CircuitTemplate(AbstractBaseTemplate):
                     if type(val) is str:
                         edge_dict[key] = val
                     else:
-                        edge_dict[key] = [val]
-                edge_dict['source_idx'] = [s_idx]
-                edge_dict['target_idx'] = [t_idx]
+                        edge_dict[key] = [val]*edge_len
+                edge_dict['source_idx'] = list(s_idx)
+                edge_dict['target_idx'] = list(t_idx)
 
                 # add edge dict to edge collection
                 edge_col[(source_new, target_new, template, delayed)] = edge_dict
@@ -1242,20 +1274,19 @@ class CircuitTemplate(AbstractBaseTemplate):
                 # add new mapping from edge source to edge input variable
                 if source == 'source':
                     source_key = orig_source
-                    idx = source_idx[i]
+                    idx = [source_idx[i]]
                 else:
                     source_key = self._relabel_var(source, label_map)
                     try:
-                        *snode, _, _ = source.split('/')
-                        idx = indices['/'.join(snode)]
+                        idx = indices[source]
                     except KeyError:
-                        idx = 0
+                        idx = [0]
 
                 # add source variable information to dict
                 try:
-                    source_dict[source_key].append(idx)
+                    source_dict[source_key].extend(idx)
                 except KeyError:
-                    source_dict[source_key] = [idx]
+                    source_dict[source_key] = list(idx)
 
             edge_sources[self._relabel_var(key, label_map)] = source_dict
 
@@ -1265,7 +1296,7 @@ class CircuitTemplate(AbstractBaseTemplate):
     def _apply_edge(edge: EdgeTemplate, values: dict, nodes: dict, label_map: dict, vectorize: bool = True):
 
         # apply edge template
-        edge_ir, label_map_tmp = edge.apply(values=values, vectorize=vectorize)  # type: EdgeIR, dict
+        edge_ir, label_map_tmp, _ = edge.apply(values=values, vectorize=vectorize)  # type: EdgeIR, dict
 
         # save edge ir references to dictionaries (treat it as a node)
         nodes[edge_ir.label] = edge_ir
