@@ -42,8 +42,8 @@ from pyrates.backend import PyRatesException, PyRatesWarning
 from pyrates.ir.node import NodeIR
 from pyrates.ir.edge import EdgeIR
 from pyrates.ir.abc import AbstractBaseIR
-from pyrates.backend.parser import parse_equations, replace, get_unique_label
-from pyrates.backend.computegraph import ComputeGraph, ComputeVar
+from pyrates.backend.parser import parse_equations, get_unique_label
+from pyrates.backend.computegraph import ComputeGraph, ComputeVar, ComputeGraphBackProp
 
 __author__ = "Daniel Rose, Richard Gast"
 __status__ = "Development"
@@ -219,17 +219,11 @@ class NetworkGraph(AbstractBaseIR):
                 # extract info from projections to input variable
                 op_name, var_name = in_var.split('/')
                 data = self._collect_from_edges(edges,
-                                                keys=['source_var', 'weight', 'delay', 'source_idx', 'target_idx'])
+                                                keys=['source_var', 'weight', 'source_idx', 'target_idx'])
 
                 # create the final equations for all edges that target the input variable
                 self._generate_edge_equation(tnode=node_name, top=op_name, tvar=var_name, inputs=data, **kwargs)
 
-        # create the final equations and variables for each edge
-        # for source, targets in self.graph.adjacency():
-        #     for target, edges in targets.items():
-        #         for idx, data in edges.items():
-        #             self._generate_edge_equation(source_node=source, target_node=target, data=data, **kwargs)
-
     def _sort_edges(self, edges: List[tuple], attr: str, data_included: bool = False) -> dict:
         """Sorts edges according to the given edge attribute.
 
@@ -332,7 +326,7 @@ class NetworkGraph(AbstractBaseIR):
             if source not in data:
                 data[source] = dict()
             for key in keys:
-                val = deepcopy(edge[key])
+                val = edge[key]  #deepcopy(edge[key])
                 try:
                     data[source][key].extend(val)
                 except AttributeError:
@@ -439,7 +433,7 @@ class NetworkGraph(AbstractBaseIR):
                                                           'shape': (len(source_idx_tmp[idx]),),
                                                           'value': source_idx_tmp[idx]}
                 elif i != 0 and idx_apply:
-                    var_prev_idx = get_indexed_var_str(var_prev, idx_str, var_length=len(rates_tmp))
+                    var_prev_idx = _get_indexed_var_str(var_prev, idx_str, var_length=len(rates_tmp))
                 else:
                     var_prev_idx = var_prev
 
@@ -498,8 +492,8 @@ class NetworkGraph(AbstractBaseIR):
             # create edge buffer variable
             buffer_length = len(delays)
             for i, idx_l, idx_r in final_idx:
-                lhs = get_indexed_var_str(f"{var}_buffered{buffer_id}", idx_l, var_length=buffer_length)
-                rhs = get_indexed_var_str(f"{var}_d{i}{buffer_id}" if i != 0 else var, idx_r, var_length=buffer_length)
+                lhs = _get_indexed_var_str(f"{var}_buffered{buffer_id}", idx_l, var_length=buffer_length)
+                rhs = _get_indexed_var_str(f"{var}_d{i}{buffer_id}" if i != 0 else var, idx_r, var_length=buffer_length)
                 buffer_eqs.append(f"{lhs} = {rhs}")
             var_dict[f"{var}_buffered{buffer_id}"] = {'vtype': 'variable',
                                                       'dtype': 'float',
@@ -670,44 +664,38 @@ class NetworkGraph(AbstractBaseIR):
             if dot_edge:
 
                 # create weight matrix
-                weight_mat = np.zeros((tsize, ssize))
-                for row, col, w in zip(tidx, sidx, weight):
+                sidx_unique = np.unique(sidx)
+                tidx_unique = np.unique(tidx)
+                weight_mat = np.zeros((len(tidx_unique), len(sidx_unique)))
+                for t, s, w in zip(tidx, sidx, weight):
+                    row = np.argwhere(tidx_unique == t).squeeze()
+                    col = np.argwhere(sidx_unique == s).squeeze()
                     weight_mat[row, col] = w
 
                 # define edge projection equation
-                eq = f"{t_str} = matvec({w_str},{s_str})"
+                s_str_final = _get_indexed_var_str(s_str, sidx_unique, ssize, idx_str=sidx_str, arg_dict=args)
+                t_str_final = _get_indexed_var_str(t_str, tidx_unique, tsize, idx_str=tidx_str, arg_dict=args)
+                eq = f"{t_str_final} = matvec({w_str}, {s_str_final})"
                 args[w_str] = {'vtype': 'constant', 'value': weight_mat, 'dtype': 'float', 'shape': weight_mat.shape}
 
             # case II: realize edge projection via source and target indexing
             else:
 
-                # check whether source variable requires indexing
-                if m > 1 or (ssize > 1 and m):
-                    ssize = len(sidx)
-                    s_str_final = f"index({s_str},{sidx_str})"
-                    args[sidx_str] = {'vtype': 'constant', 'value': sidx, 'dtype': 'int', 'shape': (ssize,)}
-                elif m == 1 and tsize > 1 and n == 1:
-                    s_str_final = f"index({s_str},{sidx[0]})"
-                    ssize = 0
-                else:
-                    s_str_final = s_str
-                    ssize = len(weight)
-
                 # check wether weighting of source variables is required
                 if all([abs(w-1) < weight_minimum for w in weight]):
                     weighting = ""
                 else:
                     weighting = f" * {w_str}"
-                    args[w_str] = {'vtype': 'constant', 'dtype': 'float', 'value': weight}
+                    args[w_str] = {'vtype': 'constant', 'dtype': 'float', 'value': weight if ssize > 1 else weight[0]}
+
+                # get final source and target strings
+                s_str_final = _get_indexed_var_str(s_str, sidx, ssize, reduce=m == 1 and tsize > 1 and n == 1,
+                                                   idx_str=sidx_str, arg_dict=args)
+                t_str_final = _get_indexed_var_str(t_str, tidx, tsize, reduce=tsize > 1 or ssize < tsize,
+                                                   idx_str=sidx_str, arg_dict=args)
 
                 # define edge equation
-                if n > 1:
-                    eq = f"index({t_str}, {tidx_str}) = {s_str_final}{weighting}"
-                    args[tidx_str] = {'vtype': 'constant', 'dtype': 'int', 'value': tidx, 'shape': (len(tidx),)}
-                elif tsize > 1 or ssize < tsize:
-                    eq = f"index({t_str}, {tidx[0]}) = {s_str_final}{weighting}"
-                else:
-                    eq = f"{t_str} = {s_str_final}{weighting}"
+                eq = f"{t_str_final} = {s_str_final}{weighting}"
 
             # add equation and source information
             eqs.append(eq)
@@ -722,7 +710,7 @@ class NetworkGraph(AbstractBaseIR):
             eqs.append(eq)
 
         # step 4: define target variable as operator output
-        args[tvar] = deepcopy(tval)
+        args[tvar] = tval  #deepcopy(tval)
         args[tvar]['vtype'] = 'variable'
 
         # step 5: add edge operator to target node
@@ -1016,7 +1004,7 @@ class CircuitIR(AbstractBaseIR):
 
         # generate run function
         func_name = kwargs.pop('func_name', 'vector_field')
-        func, func_args, _ = self.get_run_func(func_name, **kwargs)
+        func, func_args, _, _ = self.get_run_func(func_name, **kwargs)
 
         # extract backend variables that correspond to requested output variables
         if self._verbose:
@@ -1064,10 +1052,10 @@ class CircuitIR(AbstractBaseIR):
             file_name = f"pyrates_func"
         return self.graph.to_func(func_name=func_name, file_name=file_name, dt_adapt=self._dt_adapt, **kwargs)
 
-    def network_to_computegraph(self, graph: NetworkGraph, **kwargs):
+    def network_to_computegraph(self, graph: NetworkGraph, inplace_vectorfield: bool = True, **kwargs):
 
         # initialize compute graph
-        cg = ComputeGraph(**kwargs)
+        cg = ComputeGraph(**kwargs) if inplace_vectorfield else ComputeGraphBackProp(**kwargs)
 
         # add global time variable to compute graph
         cg.add_var(label="t", vtype="state_var", value=0.0 if self._dt_adapt else 0, shape=(),
@@ -1135,7 +1123,8 @@ class CircuitIR(AbstractBaseIR):
                             [op for op in ops if op_identifier in op]
                     else:
                         ops_tmp = ops
-                    op_eqs, op_vars = self._collect_ops(ops_tmp, node_name=node_name, graph=net, compute_graph=cg)
+                    op_eqs, op_vars = self._collect_ops(ops_tmp, node_name=node_name, graph=net, compute_graph=cg,
+                                                        reduce=exclude)
 
                     # parse equations and variables into computegraph
                     variables = parse_equations(op_eqs, op_vars, cg=cg, def_shape=self._def_shape, **kwargs)
@@ -1149,7 +1138,8 @@ class CircuitIR(AbstractBaseIR):
                 g.remove_nodes_from(ops)
                 i += 1
 
-    def _collect_ops(self, ops: List[str], node_name: str, graph: NetworkGraph, compute_graph: ComputeGraph) -> tuple:
+    def _collect_ops(self, ops: List[str], node_name: str, graph: NetworkGraph, compute_graph: ComputeGraph,
+                     reduce: bool) -> tuple:
         """Adds a number of operations to the backend graph.
 
         Parameters
@@ -1160,6 +1150,7 @@ class CircuitIR(AbstractBaseIR):
             Name of the node that the operators belong to.
         graph
         compute_graph
+        reduce
 
         Returns
         -------
@@ -1244,10 +1235,31 @@ class CircuitIR(AbstractBaseIR):
                         variables[full_key] = self._front_to_back[full_key]
                     except KeyError:
                         # case IV: new variables
-                        variables[full_key] = var
+                        variables[full_key] = self._finalize_var_def(var, reduce)
 
         return equations, variables
 
+    @staticmethod
+    def _finalize_var_def(v: dict, reduce: bool):
+        if not reduce:
+            return v
+        if not v:
+            return v
+        if v['vtype'] != 'constant':
+            return v
+        if v['dtype'] != 'float':
+            return v
+        if 'shape' in v and len(v['shape']) > 1:
+            return v
+        if len(np.unique(v['value'])) > 1:
+            return v
+        try:
+            v['value'] = v['value'][0]
+            v['shape'] = tuple()
+        except (TypeError, IndexError):
+            pass
+        return v
+
     @staticmethod
     def _map_multiple_inputs(inputs: dict, scope: str, tvar: str) -> tuple:
         """Creates mapping between multiple input variables and a single output variable.
@@ -1306,15 +1318,39 @@ class CircuitIR(AbstractBaseIR):
         return self.graph.edges
 
 
-def get_indexed_var_str(var: str, idx: Union[tuple, str], var_length: int = None):
+def _get_indexed_var_str(var: str, idx: Union[tuple, str, list], var_length: int = None, reduce: bool = False,
+                         idx_str: str = None, arg_dict: dict = None):
     if type(idx) is tuple:
         if var_length and int(idx[1]) - int(idx[0]) == var_length:
             return var
         elif not var_length:
             var = f"reshape({var}, 1)"
         return f"index_range({var}, {idx[0]}, {idx[1]})"
-    if idx:
+    if type(idx) is str and len(idx) > 0:
         if not var_length:
             var = f"reshape({var}, 1)"
         return f"index({var}, {idx})"
+    if len(idx) > 0:
+        if len(idx) == var_length:
+            identical = True
+            for i1, i2 in zip(idx, list(np.arange(0, var_length))):
+                if i1 != i2:
+                    identical = False
+                    break
+            if identical:
+                return var
+        if idx_str:
+            arg_dict[idx_str] = {'vtype': 'constant', 'value': idx, 'dtype': 'int', 'shape': (len(idx),)}
+            return f"index({var}, {idx_str})"
+        return f"index({var}, {idx})"
+    if reduce:
+        return f"index({var}, {idx[0]})"
     return var
+
+
+def _get_source_str():
+    pass
+
+
+def _get_target_str():
+    pass
